import iconv from "iconv-lite";
import HTML from "posthtml-parser";
import _ from "underscore";

export interface Message {
	channel: string;
	speaker: string;
	content: string;
}

export function readChatBuffer(buffer: Buffer) {
	const data = iconv.decode(buffer, "GB2312").replace(/#h.*#h/g, "");
	const content = HTML(data);
	const messages: Message[] = [];
	const curMessage: Message = {
		channel: null,
		speaker: "none",
		content: null
	}
	let hasError = false;
	for (let meta of content) {
		try {
			if (typeof (meta) === "string") {
				const messageMatch = meta.match(/^ (.*): (.*)\r\n$/);
				if (!messageMatch) {
					console.error(`Failed to parse message:`, meta);
					hasError = true;
					continue;
				}
				curMessage.speaker = messageMatch[1].length ? messageMatch[1] : "none";
				curMessage.content = messageMatch[2];
			} else if (meta.tag === "font") {
				if (!meta.content) {
					console.error(`Broken tag:`, meta);
					hasError = true;
					continue;
				}
				const messageMatch = (meta.content[0] as string).match(/^【(.{2,4})】:$/);
				if (!messageMatch) {
					console.error(`Failed to parse tag:`, meta);
					hasError = true;
					continue;
				}
				curMessage.channel = messageMatch[1];
			} else if (meta.tag === "br") {
				if (!hasError) {
					messages.push(_.clone(curMessage));
				} else {
					console.error(`Skipped message ${JSON.stringify(curMessage)} becuase it contains errors.`);
				}
				hasError = false;
			} else {
				console.error(`Unknown tag:`, meta);
				hasError = true;
			}
		} catch (e) {
			console.error(`Errored parsing tag:`, meta, e.toString());
			hasError = true;
		}
	}
	return messages;
}
